#ifndef __unitcon_h
#define __unitcon_h

class Unit;

//===========================================================================
typedef struct
{
  point   pos;            // position in world
  int     angle,          // angle relative to world in degrees
          speed;          // speed in pixels/frame
  
  int     xsize, ysize;   // size of sprites in pixels
  sprites spr;            // pointer to unit's sprite list
  int     current,        // current sprite to draw
          drawangle;      // angle to draw the sprite at
  
  int     bxsize, bysize; // half the size of the bounding box
  point   boxmin,         // opposite corners of bounding box
          boxmax;         // used for collision detection
  
  int     shields,        // strength of the unit's shields
          hull;           // strength of unit itself
          
  int     nextfire,       // unit can fire again in nextfire frames
          dead;           // flag if the unit is dead
  
  int     collision;      // damage to deal when colliding
  
  int     classname,      // what class the unit is (defined on defs.h)
          team;           // the team the unit is on
  
  Unit    *attacker;      // last unit to inflict damage on it
} UnitData;

///////////////
typedef struct
{
  Unit   *target;         // what the unit is trying to kill
} AiData;


/*
=============================================================================
  Teams:

  Teams are not actually "teams" as in game like capture the flag.
  A team is the permanent representation of a unit or group of units.
  You can't store stuff like kills or names in the unit itself, since
  it will be destroyed everytime it is killed.  A more permanent method
  is needed.  This is where teams come in.  The team array stores all of
  teams permanently throughout the game.  It is used controlled by the game
  control code for adding new players and such.  It could possibly be a
  linked list, but that seems a bit much for something so permanent.  It's
  just as easy to put new players (teams) at the end, and when something
  quits, swap the last one into it's old spot.  Sounds like this should
  be a class, but it is way too intermixed with the game control code.

  FIXME:  turn this into a class or fix it otherwise
=============================================================================
*/
typedef struct
{
  int     kills;          // how many units the team has killed
  char    name[33];       // team name (player's name)
} Team;

//===========================================================================
class Unit
{
  // FIXME:  the functions that return variables should not be neccessary?
  // it would be nice if C++ had a readonly accessor type
  public:
    Unit(UnitData &info, char *sprfile);
    virtual ~Unit();
    
    void virtual PreThink() = 0;          // control:  AI, keyboard
    void Update();                        // movement:  position
    void Check();                         // physics: collision
    void virtual PostThink() = 0;         // weapons:  AI, keyboard
    void Draw();                          // graphics:  draw it on screen
    
    void Damage(int damage, Unit *attacker, int team);
    void Killed(int team);
    void virtual TakeDamage() = 0;
    
    void NextFire(int next);
    int  Fire();

    Unit *Closest();
    Unit *ClosestNTeam();

    void SetTeam(int teamnum);

    point pos()       const { return data.pos;        }
    int   angle()     const { return data.angle;      }
    int   speed()     const { return data.speed;      }
    
    point boxmin()    const { return data.boxmin;     }
    point boxmax()    const { return data.boxmax;     }

    int   shields()   const { return data.shields;    }
    int   hull()      const { return data.hull;       }
    
    int   collision() const { return data.collision;  }

    int   team()      const { return data.team;       }
    int   classname() const { return data.classname;  }

    int   dead()      const { return data.dead;       }    

  protected:
    UnitData data;
    
    void Explode();
};

//===========================================================================
class Units
{
  public:
    Units() { last = 0; }
    ~Units() {};
    void Add(Unit *unit);
    int  Update();
    void Draw();

    Unit *ChainNext();
    Unit *ChainAll();
    Unit *ChainRadius(point origin, int radius);
    Unit *ChainTeam(int team);
    Unit *ChainNTeam(int team);
    Unit *ChainClassname(int name);
  
    int  Exist(Unit *unit);

    void Kill();
  
  protected:
    Unit *chain[2048];
    int  maxchain,
         curr;
    
    Unit *heap[2048];
    int  last;
};

//===========================================================================
const  int   TEAMS = 64;
//===========================================================================
extern Units units;
extern Team  teams[TEAMS];
extern int   numteams;

#endif
